/// <reference types="node" />
import { Envelope } from "../schema/envelope";
import { Payload } from "../schema/payload";
import * as crypto from "crypto";
export interface JsonProps {
    readonly indent: number;
    readonly newLine: string;
}
export interface SimpleEnvelopeProps<T = unknown> {
    readonly id?: string;
    readonly src: string;
    readonly dst?: string[];
    readonly t?: Date | number;
    readonly ttl?: number;
    readonly data: Payload<T>;
    readonly jsonProp?: Partial<JsonProps>;
}
declare type OutputFn = (str: string) => void;
export declare class JsonCollector {
    readonly output: OutputFn;
    readonly indent: string;
    readonly commas: string[];
    readonly elements: number[];
    readonly props: JsonProps;
    readonly nextLine: string;
    attribute?: string;
    constructor(output: OutputFn, props?: Partial<JsonProps>);
    get suffix(): string;
    append(sval: SVal): void;
}
export declare class HashCollector {
    readonly hash: crypto.Hash;
    constructor();
    digest(): string;
    append(sval: SVal): void;
}
export interface JsonHash {
    readonly jsonStr: string;
    readonly hash?: string;
}
export declare class SimpleEnvelope<T> {
    readonly simpleEnvelopeProps: SimpleEnvelopeProps<T>;
    readonly envJsonStrings: string[];
    readonly envJsonC: JsonCollector;
    envelope?: Envelope<T>;
    dataJsonHash?: JsonHash;
    constructor(env: SimpleEnvelopeProps<T>);
    asDataJson(): string;
    asJson(): string;
    asEnvelope(): Envelope<T>;
    private toDataJson;
    lazy(): this;
}
export declare function simpleEnvelope<T>(env: SimpleEnvelopeProps<T>): SimpleEnvelope<T>;
export declare function lexicalSort(a: number | string, b: number | string): number;
interface ValType {
    toString(): string;
    asValue(): any;
}
declare enum OutState {
    NONE = "NE",
    ARRAY_START = "AS",
    ARRAY_END = "AE",
    OBJECT_START = "OS",
    OBJECT_END = "OE"
}
interface SVal {
    readonly attribute?: string;
    readonly val?: ValType;
    readonly outState?: OutState;
}
declare type SValFn = (prob: SVal) => void;
export declare function sortKeys<T>(e: T, out: SValFn): void;
export {};
//# sourceMappingURL=simpleEnvelope.d.ts.map