"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortKeys = exports.lexicalSort = exports.simpleEnvelope = exports.SimpleEnvelope = exports.HashCollector = exports.JsonCollector = void 0;
var crypto = __importStar(require("crypto"));
var base_x_1 = __importDefault(require("base-x"));
var bs58 = (0, base_x_1.default)("123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz");
var JsonCollector = /** @class */ (function () {
    function JsonCollector(output, props) {
        if (props === void 0) { props = {}; }
        this.commas = [""];
        this.elements = [0];
        this.output = output;
        this.props = __assign({ indent: props.indent || 0, newLine: props.newLine || "\n" }, props);
        this.indent = Array(this.props.indent).fill(" ").join("");
        this.nextLine = this.props.indent ? this.props.newLine : "";
    }
    Object.defineProperty(JsonCollector.prototype, "suffix", {
        get: function () {
            if (this.elements[this.elements.length - 1]) {
                return (this.nextLine +
                    Array(this.commas.length - 1)
                        .fill(this.indent)
                        .join(""));
            }
            else {
                return "";
            }
        },
        enumerable: false,
        configurable: true
    });
    JsonCollector.prototype.append = function (sval) {
        if (sval.outState) {
            switch (sval.outState) {
                case OutState.ARRAY_START:
                    this.output(this.commas[this.commas.length - 1] +
                        this.suffix +
                        (this.attribute || "") +
                        "[");
                    this.attribute = undefined;
                    this.commas[this.commas.length - 1] = ",";
                    this.commas.push("");
                    this.elements.push(0);
                    break;
                case OutState.ARRAY_END:
                    this.commas.pop();
                    this.output(this.suffix + "]");
                    this.elements.pop();
                    break;
                case OutState.OBJECT_START:
                    this.output(this.commas[this.commas.length - 1] +
                        this.suffix +
                        (this.attribute || "") +
                        "{");
                    this.attribute = undefined;
                    this.commas[this.commas.length - 1] = ",";
                    this.commas.push("");
                    this.elements.push(0);
                    break;
                case OutState.OBJECT_END:
                    this.commas.pop();
                    this.output(this.suffix + "}");
                    this.elements.pop();
                    break;
            }
        }
        if (sval.val) {
            ++this.elements[this.elements.length - 1];
            var out = this.commas[this.commas.length - 1] +
                this.suffix +
                (this.attribute || "") +
                sval.val.toString();
            // console.log(this.commas, this.attribute, sval.val, out);
            this.output(out);
            this.attribute = undefined;
            this.commas[this.commas.length - 1] = ",";
            // }
        }
        if (sval.attribute) {
            ++this.elements[this.elements.length - 1];
            this.attribute =
                JSON.stringify(sval.attribute) + ":" + (this.indent.length ? " " : "");
        }
    };
    return JsonCollector;
}());
exports.JsonCollector = JsonCollector;
var HashCollector = /** @class */ (function () {
    function HashCollector() {
        this.hash = crypto.createHash("sha256");
    }
    HashCollector.prototype.digest = function () {
        return bs58.encode(this.hash.digest());
    };
    HashCollector.prototype.append = function (sval) {
        if (sval.outState) {
            return;
        }
        if (sval.attribute) {
            var tmp = Buffer.from(sval.attribute).toString('utf-8');
            // console.log('attribute=', tmp)
            this.hash.update(tmp);
        }
        if (sval.val) {
            var out = sval.val.asValue();
            if (out instanceof Date) {
                out = out.toISOString();
            }
            else {
                out = "" + out;
            }
            // console.log('val=', out)
            // We need some room for the types
            this.hash.update(Buffer.from(out).toString("utf-8"));
        }
    };
    return HashCollector;
}());
exports.HashCollector = HashCollector;
var SimpleEnvelope = /** @class */ (function () {
    function SimpleEnvelope(env) {
        var _this = this;
        this.envJsonStrings = [];
        this.simpleEnvelopeProps = env;
        this.envJsonC = new JsonCollector(function (part) { return _this.envJsonStrings.push(part); }, this.simpleEnvelopeProps.jsonProp);
    }
    SimpleEnvelope.prototype.asDataJson = function () {
        return this.dataJsonHash.jsonStr;
    };
    SimpleEnvelope.prototype.asJson = function () {
        var str = this.lazy().envJsonStrings.join("");
        // Caution
        this.asJson = function () { return str; };
        return str;
    };
    SimpleEnvelope.prototype.asEnvelope = function () {
        var _this = this;
        // Caution
        this.asEnvelope = function () { return _this.envelope; };
        return this.lazy().envelope;
    };
    SimpleEnvelope.prototype.toDataJson = function () {
        var dataJsonStrings = [];
        var dataJsonC = new JsonCollector(function (part) { return dataJsonStrings.push(part); }, __assign(__assign({}, this.simpleEnvelopeProps.jsonProp), { 
            // "data": { "data": { this stuff } } ==> 2
            newLine: "\n" +
                Array(2 * ((this.simpleEnvelopeProps.jsonProp || {}).indent || 0))
                    .fill(" ")
                    .join("") }));
        var dataHashC = undefined;
        var dataProcessor;
        if (this.simpleEnvelopeProps.id) {
            dataProcessor = function (sval) {
                dataJsonC.append(sval);
            };
        }
        else {
            dataHashC = new HashCollector();
            dataProcessor = function (sval) {
                // console.log("dataP:", sval);
                dataHashC.append(sval);
                dataJsonC.append(sval);
            };
        }
        sortKeys(this.simpleEnvelopeProps.data.data, dataProcessor);
        return {
            jsonStr: dataJsonStrings.join(""),
            hash: dataHashC ? dataHashC.digest() : undefined,
        };
    };
    SimpleEnvelope.prototype.lazy = function () {
        var _this = this;
        this.dataJsonHash = this.toDataJson();
        var t;
        if (this.simpleEnvelopeProps.t instanceof Date) {
            t = this.simpleEnvelopeProps.t.getTime();
        }
        else {
            t = this.simpleEnvelopeProps.t || (new Date()).getTime();
        }
        var envelope = {
            v: "A",
            id: this.simpleEnvelopeProps.id ||
                t + "-" + this.dataJsonHash.hash,
            src: this.simpleEnvelopeProps.src,
            dst: this.simpleEnvelopeProps.dst || [],
            t: t,
            ttl: this.simpleEnvelopeProps.ttl || 10,
            data: __assign(__assign({}, this.simpleEnvelopeProps.data), { data: undefined }),
        };
        var nextValue = false;
        sortKeys(envelope, function (sval) {
            var oval = sval;
            if (sval.attribute === "data") {
                nextValue = true;
            }
            else if (nextValue) {
                if (sval.val && sval.val.asValue() === undefined) {
                    oval = __assign(__assign({}, sval), { val: new PlainValType(_this.asDataJson()) });
                }
                nextValue = false;
            }
            _this.envJsonC.append(oval);
        });
        this.envelope = __assign(__assign({}, envelope), { data: __assign(__assign({}, envelope.data), { data: this.simpleEnvelopeProps.data.data }) });
        // Caution
        this.lazy = function () { return _this; };
        return this;
    };
    return SimpleEnvelope;
}());
exports.SimpleEnvelope = SimpleEnvelope;
function simpleEnvelope(env) {
    return new SimpleEnvelope(env);
}
exports.simpleEnvelope = simpleEnvelope;
function lexicalSort(a, b) {
    if (a < b) {
        return -1;
    }
    if (a > b) {
        return 1;
    }
    return 0;
}
exports.lexicalSort = lexicalSort;
var JsonValType = /** @class */ (function () {
    function JsonValType(val) {
        this.val = val;
    }
    JsonValType.prototype.asValue = function () {
        return this.val;
    };
    JsonValType.prototype.toString = function () {
        return JSON.stringify(this.val);
    };
    return JsonValType;
}());
var PlainValType = /** @class */ (function () {
    function PlainValType(val) {
        this.val = val;
    }
    PlainValType.prototype.asValue = function () {
        return this.val;
    };
    PlainValType.prototype.toString = function () {
        return this.val;
    };
    return PlainValType;
}());
var OutState;
(function (OutState) {
    OutState["NONE"] = "NE";
    OutState["ARRAY_START"] = "AS";
    OutState["ARRAY_END"] = "AE";
    OutState["OBJECT_START"] = "OS";
    OutState["OBJECT_END"] = "OE";
})(OutState || (OutState = {}));
function sortKeys(e, out) {
    if (Array.isArray(e)) {
        out({ outState: OutState.ARRAY_START });
        e.forEach(function (i) {
            sortKeys(i, out);
        });
        out({ outState: OutState.ARRAY_END });
        return;
    }
    else if (typeof e === "object" && !(e instanceof Date)) {
        out({ outState: OutState.OBJECT_START });
        Object.keys(e)
            .sort(lexicalSort)
            .forEach(function (i) {
            out({ attribute: i });
            sortKeys(e[i], out);
        });
        out({ outState: OutState.OBJECT_END });
        return;
    }
    else {
        out({ val: new JsonValType(e) });
        return;
    }
}
exports.sortKeys = sortKeys;
//# sourceMappingURL=simpleEnvelope.js.map