"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var simpleEnvelope_1 = require("./simpleEnvelope");
var envelope_1 = require("./lang/ts/envelope");
beforeAll(function () {
    // Lock Time
    jest.useFakeTimers("modern");
    jest.setSystemTime(new Date(1624140000000));
    // jest.spyOn(Date, 'now').mockImplementation(() => 1487076708000);
});
// it("test simple sha256 creation", () => {
//   expect.assertions(2);
//   const expectedHash =
//     "3f0a377ba0a4a460ecb616f6507ce0d8cfa3e704025d4fda3ed0c5ca05468728";
//   expect(hashIt("test message")).toEqual(expectedHash);
//   expect(hashIt("t\u0065\u0073t m\u0065\u0073\u0073ag\u0065")).toEqual(
//     expectedHash
//   );
// });
it("test simple hash", function () {
    // expect.assertions(3);
    var hashC = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({
        kind: "test",
        data: {
            name: "object",
            date: "2021-05-20",
        },
    }, function (sval) { return hashC.append(sval); });
    expect(hashC.digest()).toBe("5zWhdtvKuGob1FbW9vUGPQKobcLtYYr5wU8AxQRVraeB");
});
it("test serialization", function () {
    var message = {
        id: "1624140000000-4a2a6fb97b3afe6a7ca4c13457c441664c7f6a6c2ea7782e1f2dea384cf97cb8",
        src: "test case",
        data: {
            kind: "test",
            data: { name: "object", date: "2021-05-20" },
        },
        dst: [],
        t: new Date(444),
        ttl: 10,
    };
    var expected = {
        stringify: '{"data":{"data":{"date":"2021-05-20","name":"object"},"kind":"test"},"dst":[],"id":"1624140000000-4a2a6fb97b3afe6a7ca4c13457c441664c7f6a6c2ea7782e1f2dea384cf97cb8","src":"test case","t":444,"ttl":10,"v":"A"}',
    };
    expect((0, simpleEnvelope_1.simpleEnvelope)(message).asJson()).toEqual(expected.stringify);
});
it("test serialization with hash", function () {
    var message = {
        src: "test case",
        data: {
            kind: "test",
            data: { name: "object", date: "2021-05-20" },
        },
        dst: [],
        // t: new Date(444),
        ttl: 10,
    };
    var expected = {
        stringify: '{"data":{"data":{"date":"2021-05-20","name":"object"},"kind":"test"},"dst":[],"id":"1624140000000-BbYxQMurpUmj1W6E4EwYM79Rm3quSz1wwtNZDSsFt1bp\","src":"test case","t":1624140000000,"ttl":10,"v":"A"}',
    };
    expect((0, simpleEnvelope_1.simpleEnvelope)(message).asJson()).toEqual(expected.stringify);
});
it("test serialization with intend", function () {
    var message = {
        src: "test case",
        data: {
            kind: "test",
            data: { name: "object", date: "2021-05-20" },
        },
        dst: [],
        // t: new Date(444),
        ttl: 10,
        jsonProp: {
            indent: 2,
        },
    };
    var expected = {
        stringify: JSON.stringify(JSON.parse('{"data":{"data":{"date":"2021-05-20","name":"object"},"kind":"test"},"dst":[],"id":"1624140000000-BbYxQMurpUmj1W6E4EwYM79Rm3quSz1wwtNZDSsFt1bp","src":"test case","t":1624140000000,"ttl":10,"v":"A"}'), null, 2),
    };
    expect((0, simpleEnvelope_1.simpleEnvelope)(message).asJson()).toEqual(expected.stringify);
});
it("sort with out with string", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)("string", fn);
    expect(fn.mock.calls).toEqual([[{ val: { val: "string" } }]]);
});
it("sort with out with date", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)(new Date(444), fn);
    expect(fn.mock.calls).toEqual([[{ val: { val: new Date(444) } }]]);
});
it("sort with out with number", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)(4711, fn);
    expect(fn.mock.calls).toEqual([[{ val: { val: 4711 } }]]);
});
it("sort with out with boolean", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)(false, fn);
    expect(fn.mock.calls).toEqual([[{ val: { val: false } }]]);
});
it("sort with out with array of empty", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)([], fn);
    expect(fn.mock.calls).toEqual([[{ outState: "AS" }], [{ outState: "AE" }]]);
});
it("sort with out with array of [1,2]", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)([1, 2], fn);
    expect(fn.mock.calls).toEqual([
        [{ outState: "AS" }],
        [{ val: { val: 1 } }],
        [{ val: { val: 2 } }],
        [{ outState: "AE" }],
    ]);
});
it("sort with out with array of [[1,2],[3,4]]", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)([
        [1, 2],
        [3, 4],
    ], fn);
    expect(fn.mock.calls).toEqual([
        [{ outState: "AS" }],
        [{ outState: "AS" }],
        [{ val: { val: 1 } }],
        [{ val: { val: 2 } }],
        [{ outState: "AE" }],
        [{ outState: "AS" }],
        [{ val: { val: 3 } }],
        [{ val: { val: 4 } }],
        [{ outState: "AE" }],
        [{ outState: "AE" }],
    ]);
});
it("sort with out with obj of {}", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)({}, fn);
    expect(fn.mock.calls).toEqual([[{ outState: "OS" }], [{ outState: "OE" }]]);
});
it("sort with out with obj of { y: 1, x: 2 }", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)({ y: 1, x: 2 }, fn);
    expect(fn.mock.calls).toEqual([
        [{ outState: "OS" }],
        [{ attribute: "x" }],
        [{ val: { val: 2 } }],
        [{ attribute: "y" }],
        [{ val: { val: 1 } }],
        [{ outState: "OE" }],
    ]);
});
it("sort with out with obj of { y: { b: 1, a: 2 }  }", function () {
    var fn = jest.fn();
    (0, simpleEnvelope_1.sortKeys)({ y: { b: 1, a: 2 } }, fn);
    expect(fn.mock.calls).toEqual([
        [{ outState: "OS" }],
        [{ attribute: "y" }],
        [{ outState: "OS" }],
        [{ attribute: "a" }],
        [{ val: { val: 2 } }],
        [{ attribute: "b" }],
        [{ val: { val: 1 } }],
        [{ outState: "OE" }],
        [{ outState: "OE" }],
    ]);
});
it("JSONCollector {}", function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)({}, function (o) { return json.append(o); });
    expect(out).toBe("{}");
});
it("JSONCollector []", function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)([], function (o) { return json.append(o); });
    expect(out).toBe("[]");
});
it('JSONCollector { x: { y: 1, z: "x" }, y: {}, z: []}', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)({ x: { y: 1, z: "x" }, y: {}, z: [] }, function (o) { return json.append(o); });
    expect(out).toBe('{"x":{"y":1,"z":"x"},"y":{},"z":[]}');
});
it('JSONCollector ["xx"]', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)(["xx"], function (o) { return json.append(o); });
    expect(out).toBe('["xx"]');
});
it('JSONCollector [1, "2"]', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)([1, "2"], function (o) { return json.append(o); });
    expect(out).toBe('[1,"2"]');
});
it('JSONCollector [1, ["2", "A"]]', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); });
    (0, simpleEnvelope_1.sortKeys)([1, ["2", "A"], "E"], function (o) { return json.append(o); });
    expect(out).toBe('[1,["2","A"],"E"]');
});
it("JSONCollector indent=2 {} ", function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); }, { indent: 2 });
    (0, simpleEnvelope_1.sortKeys)({}, function (o) { return json.append(o); });
    expect(out).toBe("{}");
});
it("JSONCollector indent=2 [] ", function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); }, { indent: 2 });
    (0, simpleEnvelope_1.sortKeys)([], function (o) { return json.append(o); });
    expect(out).toBe("[]");
});
it('JSONCollector indent=2 { x: { y: 1, z: "x" }}', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); }, { indent: 2 });
    (0, simpleEnvelope_1.sortKeys)({ x: { y: 1, z: "x" }, y: {}, z: [] }, function (o) { return json.append(o); });
    expect(out).toBe('{\n  "x": {\n    "y": 1,\n    "z": "x"\n  },\n  "y": {},\n  "z": []\n}');
});
it('JSONCollector indent=2 ["xx"]', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) { return (out += o); }, { indent: 2 });
    (0, simpleEnvelope_1.sortKeys)(["xx"], function (o) { return json.append(o); });
    expect(out).toBe('[\n  "xx"\n]');
});
it('JSONCollector indent=2 [1, "2"]', function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) {
        // console.log("OUT=>", o)
        out += o;
    }, { indent: 2 });
    (0, simpleEnvelope_1.sortKeys)([1, "2"], function (o) { return json.append(o); });
    expect(out).toBe('[\n  1,\n  "2"\n]');
});
it("JSONCollector [1, new Date(444)]", function () {
    var out = "";
    var json = new simpleEnvelope_1.JsonCollector(function (o) {
        // console.log("OUT=>", o)
        out += o;
    });
    var obj = [1, new Date(444)];
    (0, simpleEnvelope_1.sortKeys)(obj, function (o) { return json.append(o); });
    expect("[1,\"1970-01-01T00:00:00.444Z\"]").toBe(out);
});
it("HashCollector 1", function () {
    var hash = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({ x: { y: 1, z: "x" }, y: {}, z: [], d: new Date(444) }, function (o) {
        return hash.append(o);
    });
    expect(hash.digest()).toBe("5PvJAWGkaKAHax6tsaKGfPYm6JfXxZs15wRTDpSKaZ2G");
});
it("HashCollector 2", function () {
    var hash = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({ x: { y: 2, z: "x" }, y: {}, z: [], date: new Date(444) }, function (o) {
        return hash.append(o);
    });
    expect(hash.digest()).toBe("ECVWfmcNaUGkgvPZe7CojrnRNULxNczKXU8PGns6UDvr");
});
it("HashCollector 3", function () {
    var hash = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({ x: { x: 1, z: "x" }, y: {}, z: [], date: new Date(444) }, function (o) {
        return hash.append(o);
    });
    expect(hash.digest()).toBe("EoYNGMtap1k9iEAGeVtHmJwpMjQLKWJmR27SG6aC9fSg");
});
it("HashCollector 4", function () {
    var hash1 = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({ x: { x: 1, z: "x" }, y: {}, z: [], date: new Date(444) }, function (o) {
        return hash1.append(o);
    });
    var hash2 = new simpleEnvelope_1.HashCollector();
    (0, simpleEnvelope_1.sortKeys)({ date: new Date(444), x: { x: 1, z: "x" }, y: {}, z: [] }, function (o) {
        return hash2.append(o);
    });
    expect(hash1.digest()).toBe(hash2.digest());
});
it('HashCollector 3 internal update', function () {
    var hashCollector = new simpleEnvelope_1.HashCollector();
    hashCollector.hash.update = jest.fn();
    (0, simpleEnvelope_1.sortKeys)({ x: { r: 1, z: "u" }, y: {}, z: [], date: new Date(444) }, function (o) { return hashCollector.append(o); });
    var result = (hashCollector.hash.update.mock.calls)
        .reduce(function (acc, _a) {
        var elem = _a[0];
        acc.push(elem.toString());
        return acc;
    }, []);
    expect(result).toEqual(["date", "1970-01-01T00:00:00.444Z", "x", "r", "1", "z", "u", "y", "z"]);
    expect(hashCollector.digest()).toBe("GKot5hBsd81kMupNCXHaqbhv3huEbxAFMLnpcX2hniwn");
});
it('missing data in envelope', function () {
    var payt = envelope_1.Convert.toPayload(JSON.stringify({
        kind: 'kind',
        data: { y: 4 }
    }));
    var message = {
        src: "test case",
        data: payt
    };
    var ref = envelope_1.Convert.toEnvelope((0, simpleEnvelope_1.simpleEnvelope)(message).asJson());
    expect((0, simpleEnvelope_1.simpleEnvelope)(ref).asEnvelope().data).toEqual(message.data);
});
afterAll(function () {
    // Unlock Time
    jest.useRealTimers();
});
//# sourceMappingURL=simpleEnvelope.test.js.map